@@ -525,7 +525,7 @@ nautilus_application_create_window (NautilusApplication *application,
 	g_return_val_if_fail (NAUTILUS_IS_APPLICATION (application), NULL);
 	nautilus_profile_start (NULL);
 
-	window = nautilus_window_new (GTK_APPLICATION (application), screen);
+	window = nautilus_window_new (screen);
 
 	maximized = g_settings_get_boolean
 		(nautilus_window_state, NAUTILUS_WINDOW_STATE_MAXIMIZED);
@@ -558,6 +558,47 @@ nautilus_application_create_window (NautilusApplication *application,
 	return window;
 }
 
+static NautilusWindowSlot *
+get_window_slot_for_location (NautilusApplication *application, GFile *location)
+{
+	NautilusWindowSlot *slot;
+	GList *l, *sl;
+
+	slot = NULL;
+
+	if (g_file_query_file_type (location, G_FILE_QUERY_INFO_NONE, NULL) != G_FILE_TYPE_DIRECTORY) {
+		location = g_file_get_parent (location);
+	} else {
+		g_object_ref (location);
+	}
+
+	for (l = gtk_application_get_windows (GTK_APPLICATION (application)); l; l = l->next) {
+		NautilusWindow *win = NAUTILUS_WINDOW (l->data);
+
+		if (NAUTILUS_IS_DESKTOP_WINDOW (win))
+			continue;
+
+		for (sl = nautilus_window_get_slots (win); sl; sl = sl->next) {
+			NautilusWindowSlot *current = NAUTILUS_WINDOW_SLOT (sl->data);
+			GFile *slot_location = nautilus_window_slot_get_location (current);
+
+			if (g_file_equal (slot_location, location)) {
+				slot = current;
+				break;
+			}
+		}
+
+		if (slot) {
+			break;
+		}
+	}
+
+	g_object_unref (location);
+
+	return slot;
+}
+
+
 static void
 open_window (NautilusApplication *application,
 	     GFile *location, GdkScreen *screen, const char *geometry)
@@ -590,6 +631,7 @@ open_window (NautilusApplication *application,
 
 static void
 open_windows (NautilusApplication *application,
+	      gboolean force_new,
 	      GFile **files,
 	      gint n_files,
 	      GdkScreen *screen,
@@ -602,8 +644,22 @@ open_windows (NautilusApplication *application,
 		open_window (application, NULL, screen, geometry);
 	} else {
 		/* Open windows at each requested location. */
-		for (i = 0; i < n_files; i++) {
-			open_window (application, files[i], screen, geometry);
+		for (i = 0; i < n_files; ++i) {
+			NautilusWindowSlot *slot = NULL;
+
+			if (!force_new)
+				slot = get_window_slot_for_location (application, files[i]);
+
+			if (!slot) {
+				open_window (application, files[i], screen, geometry);
+			} else {
+				/* We open the location again to update any possible selection */
+				nautilus_window_slot_open_location (slot, files[i], 0);
+
+				NautilusWindow *window = nautilus_window_slot_get_window (slot);
+				nautilus_window_set_active_slot (window, slot);
+				gtk_window_present (GTK_WINDOW (window));
+			}
 		}
 	}
 }
@@ -615,19 +671,28 @@ nautilus_application_open_location (NautilusApplication *application,
 				    const char *startup_id)
 {
 	NautilusWindow *window;
+	NautilusWindowSlot *slot;
 	GList *sel_list = NULL;
 
 	nautilus_profile_start (NULL);
 
-	window = nautilus_application_create_window (application, gdk_screen_get_default ());
-	gtk_window_set_startup_id (GTK_WINDOW (window), startup_id);
+	slot = get_window_slot_for_location (application, location);
+
+	if (!slot) {
+		window = nautilus_application_create_window (application, gdk_screen_get_default ());
+		slot = nautilus_window_get_active_slot (window);
+	} else {
+		window = nautilus_window_slot_get_window (slot);
+		nautilus_window_set_active_slot (window, slot);
+		gtk_window_present (GTK_WINDOW (window));
+	}
 
 	if (selection != NULL) {
 		sel_list = g_list_prepend (sel_list, nautilus_file_get (selection));
 	}
 
-	nautilus_window_slot_open_location_full (nautilus_window_get_active_slot (window), location,
-						 0, sel_list, NULL, NULL);
+	gtk_window_set_startup_id (GTK_WINDOW (window), startup_id);
+	nautilus_window_slot_open_location_full (slot, location, 0, sel_list, NULL, NULL);
 
 	if (sel_list != NULL) {
 		nautilus_file_list_free (sel_list);
@@ -646,7 +711,9 @@ nautilus_application_open (GApplication *app,
 
 	DEBUG ("Open called on the GApplication instance; %d files", n_files);
 
-	open_windows (self, files, n_files,
+	gboolean force_new = (g_strcmp0 (hint, "new-window") == 0);
+
+	open_windows (self, force_new, files, n_files,
 		      gdk_screen_get_default (),
 		      self->priv->geometry);
 }
@@ -874,6 +941,9 @@ action_kill (GSimpleAction *action,
 	/* this will also destroy the desktop windows */
 	windows = gtk_application_get_windows (application);
 	g_list_foreach (windows, (GFunc) gtk_widget_destroy, NULL);
+
+	/* we have been asked to force quit */
+	g_application_quit (G_APPLICATION (application));
 }
 
 static void
@@ -1137,6 +1207,7 @@ nautilus_application_local_command_line (GApplication *application,
 	gboolean version = FALSE;
 	gboolean browser = FALSE;
 	gboolean kill_shell = FALSE;
+	gboolean open_new_window = FALSE;
 	gboolean no_default_window = FALSE;
 	gboolean select_uris = FALSE;
 	gchar **remaining = NULL;
@@ -1154,6 +1225,8 @@ nautilus_application_local_command_line (GApplication *application,
 		  N_("Show the version of the program."), NULL },
 		{ "geometry", 'g', 0, G_OPTION_ARG_STRING, &self->priv->geometry,
 		  N_("Create the initial window with the given geometry."), N_("GEOMETRY") },
+		{ "new-window", 'w', 0, G_OPTION_ARG_NONE, &open_new_window,
+		  N_("Always open a new window for browsing specified URIs"), NULL },
 		{ "no-default-window", 'n', 0, G_OPTION_ARG_NONE, &no_default_window,
 		  N_("Only create windows for explicitly specified URIs."), NULL },
 		{ "no-desktop", '\0', 0, G_OPTION_ARG_NONE, &self->priv->no_desktop,
@@ -1172,7 +1245,6 @@ nautilus_application_local_command_line (GApplication *application,
 	GError *error = NULL;
 	gint argc = 0;
 	gchar **argv = NULL;
-
 	*exit_status = EXIT_SUCCESS;
 
 	nautilus_profile_start (NULL);
@@ -1210,9 +1282,9 @@ nautilus_application_local_command_line (GApplication *application,
 		goto out;
 	}
 
-	DEBUG ("Parsing local command line, no_default_window %d, quit %d, "
-	       "self checks %d, no_desktop %d, show_desktop %d",
-	       no_default_window, kill_shell, perform_self_check,
+	DEBUG ("Parsing local command line: open_new_window %d, no_default_window %d, "
+	       "quit %d, self checks %d, no_desktop %d, show_desktop %d",
+	       open_new_window, no_default_window, kill_shell, perform_self_check,
 	       self->priv->no_desktop, self->priv->force_desktop);
 
 	g_application_register (application, NULL, &error);
@@ -1275,7 +1347,7 @@ nautilus_application_local_command_line (GApplication *application,
 		nautilus_application_select (self, files, len);
 	} else {
 		/* Invoke "Open" to create new windows */
-		g_application_open (application, files, len, "");
+		g_application_open (application, files, len, open_new_window ? "new-window" : "");
 	}
 
 	for (idx = 0; idx < len; idx++) {
@@ -1567,11 +1639,11 @@ nautilus_application_startup (GApplication *app)
 	 * if there are problems.
 	 */
 	check_required_directories (self);
-	init_desktop (self);
 
 	do_upgrades_once (self);
 
 	nautilus_application_init_actions (self);
+	init_desktop (self);
 
 	nautilus_profile_end (NULL);
 }
@@ -1589,6 +1661,99 @@ nautilus_application_quit_mainloop (GApplication *app)
 	G_APPLICATION_CLASS (nautilus_application_parent_class)->quit_mainloop (app);
 }
 
+static void
+update_dbus_opened_locations (NautilusApplication *app)
+{
+	gint i;
+	GList *l, *sl;
+	GList *locations = NULL;
+	gsize locations_size = 0;
+	gchar **locations_array;
+
+	g_return_if_fail (NAUTILUS_IS_APPLICATION (app));
+
+	for (l = gtk_application_get_windows (GTK_APPLICATION (app)); l; l = l->next) {
+		NautilusWindow *win = NAUTILUS_WINDOW (l->data);
+
+		if (NAUTILUS_IS_DESKTOP_WINDOW (win)) {
+			continue;
+		}
+
+		for (sl = nautilus_window_get_slots (win); sl; sl = sl->next) {
+			NautilusWindowSlot *slot = NAUTILUS_WINDOW_SLOT (sl->data);
+			gchar *uri = nautilus_window_slot_get_location_uri (slot);
+
+			if (uri) {
+				GList *found = g_list_find_custom (locations, uri, (GCompareFunc) g_strcmp0);
+
+				if (!found) {
+					locations = g_list_prepend (locations, uri);
+					++locations_size;
+				} else {
+					g_free (uri);
+				}
+			}
+		}
+	}
+
+	locations_array = g_new (gchar*, locations_size + 1);
+
+	for (i = 0, l = locations; l; l = l->next, ++i) {
+		/* We reuse the locations string locations saved on list */
+		locations_array[i] = l->data;
+	}
+
+	locations_array[locations_size] = NULL;
+
+	nautilus_freedesktop_dbus_set_open_locations (app->priv->fdb_manager,
+		                                      (const gchar**) locations_array);
+
+	g_free (locations_array);
+	g_list_free_full (locations, g_free);
+}
+
+static void
+on_slot_location_changed (NautilusWindowSlot *slot,
+			  const char         *from,
+ 			  const char         *to,
+			  NautilusApplication *application)
+{
+	update_dbus_opened_locations (application);
+}
+
+static void
+on_slot_added (NautilusWindow      *window,
+	       NautilusWindowSlot  *slot,
+	       NautilusApplication *application)
+{
+	if (nautilus_window_slot_get_location (slot)) {
+		update_dbus_opened_locations (application);
+	}
+
+	g_signal_connect (slot, "location-changed", G_CALLBACK (on_slot_location_changed), application);
+}
+
+static void
+on_slot_removed (NautilusWindow      *window,
+		 NautilusWindowSlot  *slot,
+		 NautilusApplication *application)
+{
+	update_dbus_opened_locations (application);
+
+	g_signal_handlers_disconnect_by_func (slot, on_slot_location_changed, application);
+}
+
+static void
+nautilus_application_window_added (GtkApplication *app,
+				   GtkWindow *window)
+{
+	/* chain to parent */
+	GTK_APPLICATION_CLASS (nautilus_application_parent_class)->window_added (app, window);
+
+	g_signal_connect (window, "slot-added", G_CALLBACK (on_slot_added), app);
+	g_signal_connect (window, "slot-removed", G_CALLBACK (on_slot_removed), app);
+}
+
 static void
 nautilus_application_window_removed (GtkApplication *app,
 				     GtkWindow *window)
@@ -1603,6 +1768,9 @@ nautilus_application_window_removed (GtkApplication *app,
 		previewer = nautilus_previewer_get_singleton ();
 		nautilus_previewer_call_close (previewer);
 	}
+
+	g_signal_handlers_disconnect_by_func (window, on_slot_added, app);
+	g_signal_handlers_disconnect_by_func (window, on_slot_removed, app);
 }
 
 static void
@@ -1622,7 +1790,19 @@ nautilus_application_class_init (NautilusApplicationClass *class)
 	application_class->local_command_line = nautilus_application_local_command_line;
 
 	gtkapp_class = GTK_APPLICATION_CLASS (class);
+	gtkapp_class->window_added = nautilus_application_window_added;
 	gtkapp_class->window_removed = nautilus_application_window_removed;
 
 	g_type_class_add_private (class, sizeof (NautilusApplicationPriv));
 }
+
+NautilusApplication *
+nautilus_application_new (void)
+{
+	return g_object_new (NAUTILUS_TYPE_APPLICATION,
+			     "application-id", "org.gnome.Nautilus",
+			     "flags", G_APPLICATION_HANDLES_OPEN,
+			     "inactivity-timeout", 12000,
+			     "register-session", TRUE,
+			     NULL);
+}
