@@ -99,27 +99,27 @@ canvas_container_set_workarea (NautilusCanvasContainer *canvas_container,
 			     int                    n_items)
 {
 	int left, right, top, bottom;
+	int screen_width, screen_height;
 	int i;
-	GdkRectangle geometry;
 
 	left = right = top = bottom = 0;
-	gdk_screen_get_monitor_geometry (screen, gdk_screen_get_primary_monitor (screen), &geometry);
+
+	screen_width  = gdk_screen_get_width (screen);
+	screen_height = gdk_screen_get_height (screen);
 
 	for (i = 0; i < n_items; i += 4) {
-		GdkRectangle workarea;
+		int x      = workareas [i];
+		int y      = workareas [i + 1];
+		int width  = workareas [i + 2];
+		int height = workareas [i + 3];
 
-		workarea.x = workareas[i];
-		workarea.y = workareas[i + 1];
-		workarea.width = workareas[i + 2];
-		workarea.height = workareas[i + 3];
-
-		if (!gdk_rectangle_intersect (&geometry, &workarea, &workarea))
+		if ((x + width) > screen_width || (y + height) > screen_height)
 			continue;
 
-		left   = MAX (left, workarea.x);
-		right  = MAX (right, (geometry.x + geometry.width) - (workarea.x + workarea.width));
-		top    = MAX (top, workarea.y);
-		bottom = MAX (bottom, (geometry.y + geometry.height) - (workarea.y + workarea.height));
+		left   = MAX (left, x);
+		right  = MAX (right, screen_width - width - x);
+		top    = MAX (top, y);
+		bottom = MAX (bottom, screen_height - height - y);
 	}
 
 	nautilus_canvas_container_set_margins (canvas_container,
@@ -655,7 +655,7 @@ real_update_menus (NautilusView *view)
 
 static const GtkActionEntry desktop_view_entries[] = {
 	/* name, stock id */
-	{ "Change Background", NULL,
+	{ NAUTILUS_ACTION_CHANGE_BACKGROUND, NULL,
 	  /* label, accelerator */
 	  N_("Change Desktop _Background"), NULL,
 	  /* tooltip */
@@ -697,6 +697,8 @@ real_merge_menus (NautilusView *view)
 	NautilusDesktopCanvasView *desktop_view;
 	GtkUIManager *ui_manager;
 	GtkActionGroup *action_group;
+	GtkAction *action;
+	gchar *control_center_path;
 
 	NAUTILUS_VIEW_CLASS (nautilus_desktop_canvas_view_parent_class)->merge_menus (view);
 
@@ -731,6 +733,14 @@ real_merge_menus (NautilusView *view)
 			       NAUTILUS_ACTION_UNSTRETCH,
 			       GTK_UI_MANAGER_MENUITEM,
 			       FALSE);
+
+	control_center_path = g_find_program_in_path ("gnome-control-center");
+	if (control_center_path == NULL) {
+		action = gtk_action_group_get_action (action_group, NAUTILUS_ACTION_CHANGE_BACKGROUND);
+		gtk_action_set_visible (action, FALSE);
+	}
+
+	g_free (control_center_path);
 }
 
 static NautilusView *
